unit Codes;

interface

uses Globals;

type
  TCodeType = (ctUnknown,ctEAN8,ctUPC12,ctEAN13,ctITF14);
  TCodeDescription = (cdUnknown,cdBad,cdOk);
  TCheckSumDescription = (csdUnknown,csdOk);
  TRegion = record
    Name: string;
    Description: TCodeDescription;
  end;
  TCheckSum = record
    CheckSum: string;
    Description: TCheckSumDescription;
  end;
  TUsingArea = record
    Name: string;
    Description: TCodeDescription;
  end;
  TCodeFullDescription = record
    sCode: string;             //  
    sCodeLength: string;       //  
    eCodeType: TCodeType;      //  
    sCodeType: string;         //   
    sCodeUsing: string;        //   ( )
    sCodeRegion: string;       // ,    
    sCodeCheckSum: string;     //  ,   
    sCodeCalcCheckSum: string; //   
    eCodeDescription: TCodeDescription;
    sCodeDeduction: string;    //   
  end;

//  
function CodeType(const s: string): TCodeType;
function CodeTypeStr(const s: string): string;
//   
function CodeCalcCheckSum(const Code: string; const CodeType: TCodeType): TCheckSum;
//  ,   
function CodeCheckSum(const Code: string; const CodeType: TCodeType): TCheckSum;
//   
function CodeRegion(const Code: string; const CodeType: TCodeType): TRegion;
//     ( )
function CodeUsing(const Code: string; const CodeType: TCodeType): TUsingArea;
//   
function CodeFullDescription(const Code: string): TCodeFullDescription;

implementation

//    ,  .  .
//  - . EAN_UPC_ITF_Country

const
  //    
  sUnknownCheckSum = '   ';
  sUnknownCountry = '   ';
  sUnknownCode = ' ';
  sUnknown = '';
  sReservedCode = '  ';
  sBadCode = ' ';
  sUSA = '  ';
  sOR = '  ';
  // 
  sUnknownProduct = '    ()';
  sBadProduct = '!  ';
  sStrangeProduct = ',  ';
  sGoodProduct = ',    ';
  //   
  sError = '   .  .';

const
  //   ( )
  //      , 
  //      (. ITF14CodeUsing)
  UnknownCodeUsing = sUnknownCode;
  EAN8CodeUsing = '  ';
  UPC12CodeUsing = '      ';
  EAN13CodeUsing = UPC12CodeUsing;

  //   
  ITF14_1_CodeUsing = '  -  ';
  ITF14_2_CodeUsing = '  - ';
  ITF14_3to8_CodeUsing = '  -   ';
  ITF14_BAD_CodeUsing = '   !';

{
 ========================================================================
 ========================    =========================
}
function CodeType;
begin
  Result := ctUnknown;
  if AllDigits(s) then begin
    case Length(s) of
      8:  Result := ctEAN8;
      12: Result := ctUPC12;
      13: Result := ctEAN13;
      14: Result := ctITF14;
    end; // case
  end; // if
end;

function CodeTypeStr;
begin
  case CodeType(s) of
    ctEAN8: Result := 'EAN-8';
    ctUPC12: Result := 'UPC-12';
    ctEAN13: Result := 'EAN-13'+sOR+'UPC-13';
    ctITF14: Result := 'ITF-14';
    else Result := sUnknownCode;
  end;
end;
{
 ================     ========================
 ========================================================================
}

{
 ========================================================================
 ================    ===========================
}
//    :
// !!!      25 () !!!
// 1 -  ,    
// 2 -     3
// 3 -     ,   
// 4 -  ,    2  3
// 5 -  
// 6 -   10     5
//     EAN-8, UPC-12, EAN-13, ITF-14
function EANCalcCheckSumStr(const Code: string): char;
const // max  EAN/UPC :   ר
  l = 25;
var
  n: array[1..l] of byte;
  i,k,r: integer;
begin
  FillChar(n,SizeOf(n),#0);
  k := l - Length(Code);
  //     
  for i := 1 to Length(Code) do n[k+i] := byte(Code[i])-byte('0');
  i := 0; r := 0;
  while i < l-1 do begin
    Inc(i,2);
    r := r + n[i] + n[i] + n[i] + n[i-1];
  end;
  Result := Char( ((10 - (r mod 10)) mod 10)+byte('0') );
end;

function CodeCalcCheckSum;
begin
  case CodeType of
    ctEAN8,ctUPC12,ctEAN13,ctITF14:
      begin
        Result.CheckSum := EANCalcCheckSumStr(Code);
        Result.Description := csdOk;
      end;
    else
      begin
        Result.CheckSum := sUnknownCheckSum;
        Result.Description := csdUnknown;
      end;
  end; // case
end;

function EANCheckSumStr(const Code: string): char;
begin
  Result := Code[Length(Code)];
end;

function CodeCheckSum;
begin
  case CodeType of
    ctEAN8,ctUPC12,ctEAN13,ctITF14:
      begin
        Result.CheckSum := EANCheckSumStr(Code);
        Result.Description := csdOk;
      end;
    else
      begin
        Result.CheckSum := sUnknownCheckSum;
        Result.Description := csdUnknown;
      end;
  end; // case
end;
{
 ================     ==================
 ========================================================================
}

{
 ========================================================================
 ================   ================================
}
function ITF14CodeUsing(const Code: string): TUsingArea;
begin
  Result.Description := cdOk;
  case StrToInt(Code[1]) of
    1: Result.Name := ITF14_1_CodeUsing;
    2: Result.Name := ITF14_2_CodeUsing;
    3..8: Result.Name := ITF14_3to8_CodeUsing + Code[1];
    else begin
      Result.Name := ITF14_Bad_CodeUsing;
      Result.Description := cdBad;
    end;
  end;
end;

function CodeUsing;
begin
  Result.Description := cdOk;
  case CodeType of
    ctEAN8:  Result.Name := EAN8CodeUsing;
    ctUPC12: Result.Name := UPC12CodeUsing;
    ctEAN13: Result.Name := EAN13CodeUsing;
    ctITF14: Result := ITF14CodeUsing(Code);
    else begin
      Result.Name := sUnknown;
      Result.Description := cdUnknown;
    end;
  end;
end;
{
 ================    =======================
 ========================================================================
}

{
 ========================================================================
 ========================  ========================================
}
//   -  (3 )
//   - ,    
function EAN_UPC_ITF_Country(const Code: string): TRegion;
begin
  Result.Description := cdOk;
  case StrToInt(Code) of
    0..139: Result.Name := sUSA;
    200..299:
      begin
        Result.Name := sReservedCode;
        Result.Description := cdBad;
      end;
    300..379: Result.Name := '';
    380: Result.Name := '';
    383: Result.Name := '';
    385: Result.Name := '';
    387: Result.Name := '-';
    400..440: Result.Name := '';
    450..459: Result.Name := '';
    460: Result.Name := '';
    461..469:
      begin
        Result.Name := sReservedCode;
        Result.Description := cdBad;
      end;
    470: Result.Name := '';
    471: Result.Name := '';
    474: Result.Name := '';
    475: Result.Name := '';
    476: Result.Name := '';
    477: Result.Name := '';
    478: Result.Name := '';
    479: Result.Name := '-';
    480: Result.Name := '';
    481: Result.Name := '';
    482: Result.Name := '';
    484: Result.Name := '';
    485: Result.Name := '';
    486: Result.Name := '';
    487: Result.Name := '';
    489: Result.Name := '';
    490..499: Result.Name := '';
    500..509: Result.Name := '';
    520: Result.Name := '';
    528: Result.Name := '';
    529: Result.Name := '';
    531: Result.Name := '';
    535: Result.Name := '';
    539: Result.Name := '';
    540..549: Result.Name := '  ';
    560: Result.Name := '';
    569: Result.Name := '';
    570..579: Result.Name := '';
    590: Result.Name := '';
    594: Result.Name := '';
    599: Result.Name := '';
    600..601: Result.Name := ' ';
    608: Result.Name := '';
    609: Result.Name := '';
    611: Result.Name := '';
    613: Result.Name := '';
    616: Result.Name := '';
    619: Result.Name := '';
    621: Result.Name := '';
    622: Result.Name := '';
    624: Result.Name := '';
    625: Result.Name := '';
    626: Result.Name := '';
    627: Result.Name := '';
    628: Result.Name := ' ';
    629: Result.Name := '  ';
    640..649: Result.Name := '';
    690..693: Result.Name := '';
    700..709: Result.Name := '';
    729: Result.Name := '';
    730..739: Result.Name := '';
    740: Result.Name := '';
    741: Result.Name := '';
    742: Result.Name := '';
    743: Result.Name := '';
    744: Result.Name := '-';
    745: Result.Name := '';
    746: Result.Name := ' ';
    750: Result.Name := '';
    759: Result.Name := '';
    760..769: Result.Name := '';
    770: Result.Name := '';
    773: Result.Name := '';
    775: Result.Name := '';
    777: Result.Name := '';
    779: Result.Name := '';
    780: Result.Name := '';
    784: Result.Name := '';
    786: Result.Name := '';
    789: Result.Name := '';
    800..839: Result.Name := '';
    840..849: Result.Name := '';
    850: Result.Name := '';
    858: Result.Name := '';
    859: Result.Name := '';
    860: Result.Name := '';
    867: Result.Name := ' ';
    869: Result.Name := '';
    870..879: Result.Name := '';
    880: Result.Name := ' ';
    885: Result.Name := '';
    888: Result.Name := '';
    890: Result.Name := '';
    893: Result.Name := '';
    899: Result.Name := '';
    900..919: Result.Name := '';
    930..939: Result.Name := '';
    940..949: Result.Name := ' ';
    955: Result.Name := '';
    958: Result.Name := '';
    977: Result.Name := ' -   (ISSN)';
    978: Result.Name := ' -  (ISBN)';
    979: Result.Name := ' -    (ISBN)';
    980: Result.Name := ' -  ';
    981..982: Result.Name := ' -  ';
    990..999: Result.Name := ' -   ';
    else begin
      Result.Name := sUnknownCountry;
      Result.Description := cdUnknown;
    end;
  end; // case
end;

function EANCountry(const Code: string): TRegion;
begin
  if EANCheckSumStr(Code) <> EANCalcCheckSumStr(Code) then begin
    Result.Name := sBadCode;
    Result.Description := cdBad;
    end
  else Result := EAN_UPC_ITF_Country(Code[1]+Code[2]+Code[3]);
end;

function UPC12Country(const Code: string): TRegion;
begin
  if EANCheckSumStr(Code) = EANCalcCheckSumStr(Code) then begin
    Result.Name := sUSA;
    Result.Description := cdOk;
    end
  else begin
    Result.Name := sBadCode;
    Result.Description := cdBad;
  end;
end;

function ITF14Country(const Code: string): TRegion;
begin
  if EANCheckSumStr(Code) <> EANCalcCheckSumStr(Code) then begin
    Result.Name := sBadCode;
    Result.Description := cdBad;
    end
  else Result := EAN_UPC_ITF_Country(Code[2]+Code[3]+Code[4]);
end;

function CodeRegion;
begin
  Result.Name := sUnknown;
  Result.Description := cdUnknown;
  case CodeType of
    ctEAN8,ctEAN13:
      begin
        Result := EANCountry(Code);
      end;
    ctUPC12:
      begin
        Result := UPC12Country(Code);
      end;
    ctITF14:
      begin
        Result := ITF14Country(Code);
      end;
  end; // case
end;
{
 ========================   ===============================
 ========================================================================
}


function CodeFullDescription;
var
  cs1,cs2: TCheckSum;
  ua: TUsingArea;
  cr: TRegion;
begin
 //   ,   
  Result.sCode := Code;
  Result.sCodeLength := IntToStr(Length(Code));
  Result.eCodeType := CodeType(Code);
  Result.sCodeType := CodeTypeStr(Code);

 //     Description  Deduction
 // -     
  Result.eCodeDescription := cdUnknown;
  Result.sCodeDeduction := sError;

 //  ,    
  cs1 := CodeCheckSum(Code,Result.eCodeType);
  cs2 := CodeCalcCheckSum(Code,Result.eCodeType);
  ua  := CodeUsing(Code,Result.eCodeType);
  cr  := CodeRegion(Code,Result.eCodeType);

  Result.sCodeUsing := ua.Name;
  Result.sCodeRegion := cr.Name;
  Result.sCodeCheckSum := cs1.CheckSum;
  Result.sCodeCalcCheckSum := cs2.CheckSum;

 // 

  //   
  if Result.eCodeType = ctUnknown then begin
    Result.eCodeDescription := cdUnknown;
    Result.sCodeDeduction := sUnknownCode;
    Exit;
  end;

  //   
  if cs1.CheckSum <> cs2.CheckSum then begin
    Result.eCodeDescription := cdBad;
    Result.sCodeDeduction := sBadProduct;
    Exit;
  end;

  //   
  if ua.Description = cdBad then begin
    Result.eCodeDescription := cdBad;
    Result.sCodeDeduction := sBadProduct;
    Exit;
  end;

  //   
  if cr.Description = cdBad then begin
    Result.eCodeDescription := cdBad;
    Result.sCodeDeduction := sBadProduct;
    Exit;
  end;

  //     
  if cr.Description = cdUnknown then begin
    Result.eCodeDescription := cdUnknown;
    Result.sCodeDeduction := sStrangeProduct;
    Exit;
  end;

  //   
  Result.eCodeDescription := cdOk;
  Result.sCodeDeduction := sGoodProduct;
end;

end.
