unit Main;

interface

uses
  Forms, StdCtrls, Controls, Classes,
  //
  Globals, Codes, About;

type
  TMainForm = class(TForm)
    ExitBtn: TButton;
    Memo: TMemo;
    AboutBtn: TButton;
    BeepCheckBox: TCheckBox;
    CodeEditLabel: TLabel;
    CodeEdit: TEdit;
    BadCodeBeepCheckBox: TCheckBox;
    procedure ExitBtnClick(Sender: TObject);
    procedure AboutBtnClick(Sender: TObject);
    procedure CodeEditChange(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

const
  sCrLf = #13#10;
  sCode = '%1';
  sCodeLength = '%2';
  sCodeType = '%3';
  sCodeUsed = '%4';
  sCodeRegion = '%5';
  sCodeCheckSum = '%6';
  sCodeCalcCheckSum = '%7';
  sCodeDeduction = '%8';

  CodeDescText =
    '-: ' + sCode + sCrLf +
    ' , : ' + sCodeLength + sCrLf +
    ' : ' + sCodeType + sCrLf +
    ' : ' + sCodeUsed + sCrLf +
    '  : ' + sCodeRegion + sCrLf +
    '  (): ' + sCodeCheckSum + sCrLf +
    '  (): ' + sCodeCalcCheckSum + sCrLf +
    ': ' + sCodeDeduction;

procedure TMainForm.ExitBtnClick(Sender: TObject);
begin
  MainForm.Close;
end;

procedure DoBeep(const CodeFullDescription: TCodeFullDescription);
begin
  with CodeFullDescription do begin
    //   - 
    if eCodeType = ctUnknown then Exit;
    //   
    with MainForm do if BeepCheckBox.Checked then begin
      if not BadCodeBeepCheckBox.Checked then MessageBeep(MB_DEFAULT)
      else begin
        if eCodeDescription = cdOk then MessageBeep(MB_OK) else MessageBeep(MB_ICONEXCLAMATION);
      end;
    end;
  end;
end;

procedure TMainForm.CodeEditChange(Sender: TObject);
var
  cd: TCodeFullDescription;
  t: string;
begin
  t := CodeDescText;
  cd := CodeFullDescription(CodeEdit.Text);
  ChangeChars(t,sCode,cd.sCode);
  ChangeChars(t,sCodeLength,cd.sCodeLength);
  ChangeChars(t,sCodeType,cd.sCodeType);
  ChangeChars(t,sCodeUsed,cd.sCodeUsing);
  ChangeChars(t,sCodeRegion,cd.sCodeRegion);
  ChangeChars(t,sCodeCheckSum,cd.sCodeCheckSum);
  ChangeChars(t,sCodeCalcCheckSum,cd.sCodeCalcCheckSum);
  ChangeChars(t,sCodeDeduction,cd.sCodeDeduction);
  Memo.Text := t;
  DoBeep(cd);
end;

procedure TMainForm.AboutBtnClick(Sender: TObject);
begin
  with AboutForm do begin
    Left := MainForm.Left + ExitBtn.Left - Width + ExitBtn.Width;
    Top := MainForm.Top + AboutBtn.Top + AboutBtn.Height - Height + 10;
    AboutForm.ShowModal;
  end;
end;

// ,   <Esc>
//    ,      
procedure TMainForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = Char(ESC) then Halt(0);
  with CodeEdit do begin
    if (Key <> Char(ENTER)) and not Focused then begin
      Text := Text + Key;
      SetFocus;
      SelStart := Length(Text);
      SelLength := 0;
    end;
  end;
end;

end.
